@@ -13,10 +13,9 @@ test_that("can add an intercept column", {
 test_that("existing intercepts are skipped with a warning", {
   x <- add_intercept_column(mtcars)
 
-  expect_warning(
-    xx <- add_intercept_column(x),
-    "`data` already has a column named"
-  )
+  expect_snapshot({
+    xx <- add_intercept_column(x)
+  })
 
   expect_equal(
     xx,
@@ -31,13 +30,10 @@ test_that("can change the intercept column name", {
 })
 
 test_that("name can only be a single character", {
-  expect_error(
-    add_intercept_column(mtcars, name = c("x", "y")),
-    "name should have size 1, not 2."
-  )
-
-  expect_error(
-    add_intercept_column(mtcars, name = 1),
-    "name should be a character, not a numeric."
-  )
+  expect_snapshot(error = TRUE, {
+    add_intercept_column(mtcars, name = c("x", "y"))
+  })
+  expect_snapshot(error = TRUE, {
+    add_intercept_column(mtcars, name = 1)
+  })
 })
